/*
    SPDX-FileCopyrightText: 2010 Grégory Oestreicher <greg@kamago.net>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "davmultigetprotocol_p.h"

using namespace KDAV;

DavMultigetProtocol::~DavMultigetProtocol()
{
}
